/*{ Template_FreeRTOS::.c ************************************************

     Several different boxes run over the screen.
     Each box is controlled by one task.
     
}*/

#include "example_lcd_boxes.h"

u16_t LCD_Width  = 240;
u16_t LCD_Height = 320;
u16_t Color_Background = LCD_COLOR_YELLOW;

//{ Function definitions *************************************************

void lcd_boxes_initPorts() {

    //{ init_LCD Start
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    GPIO_Init(GPIOD, &GPIO_InitStructure);
    GPIO_Init(GPIOE, &GPIO_InitStructure);
    GPIO_Init(GPIOF, &GPIO_InitStructure);
    GPIO_Init(GPIOG, &GPIO_InitStructure);

    /* Enable the FSMC Clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    LCD_Init();

    //} init_LCD Ende

    LCD_SetBacklight(1);
    LCD_Clear(Color_Background);

}
void lcd_start_boxes() {

    xQueueHandle DrawQueue = xQueueCreate(10, sizeof(box2d_t));
    Assert(NULL != DrawQueue, ec_Malloc);

    lcd_boxes_initPorts();

    if (1) {
        box2d_t* Box = newBox(DrawQueue, LCD_COLOR_BLUE, 1, 0, 15, 15, 4, 5);
        Assert(0 != Box, ec_Malloc);
    }
    if (1) {
        box2d_t* Box = newBox(DrawQueue, LCD_COLOR_RED, 50, 100, 6, 9, -4, 6);
        Assert(0 != Box, ec_Malloc);
    }
    if (1) {
        box2d_t* Box = newBox(DrawQueue, LCD_COLOR_GREEN, 100, 200, 15, 10, 7, -6);
        Assert(0 != Box, ec_Malloc);
    }
    if (1) {
        box2d_t* Box = newBox(DrawQueue, LCD_COLOR_GRAY, 0, 300, 6, 6, -2, -3);
        Assert(0 != Box, ec_Malloc);
    }

    portBASE_TYPE Task2 = xTaskCreate(tDrawRect,                          // function to start as thread
                                      (const signed char*) "DrawsRect",        // thread name (just for debugging)
                                      (size_t) 1 * configMINIMAL_STACK_SIZE,                      // stack size
                                      (void *) DrawQueue,              // passed as argument to taskLEDs()
                                      (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                                      (xTaskHandle*) NULL                // can return a handle to created task
                                      );
    Assert(0 != Task2, ec_Malloc);
}
box2d_t* newBox(xQueueHandle DrawingQueue, u16_t Color, s16_t X, s16_t Y, u8_t Width, u8_t Height, s8_t SpeedX, s8_t SpeedY) {
    drawTaskArgs_t* NewBox = (drawTaskArgs_t*) pvPortMalloc( sizeof(drawTaskArgs_t) );

    if (NewBox != NULL) {
        NewBox->DrawQueue = DrawingQueue;
        NewBox->Box2Draw.BoxColor   = Color;
        NewBox->Box2Draw.PosX  = X;
        NewBox->Box2Draw.PosY  = Y;
        NewBox->Box2Draw.Width = Width;
        NewBox->Box2Draw.Height = Height;
        NewBox->Box2Draw.SpeedX = SpeedX;
        NewBox->Box2Draw.SpeedY = SpeedY;

        portBASE_TYPE Task = xTaskCreate(tMoveBox,                             // function to start as thread
                                         (const signed char*) NULL,             // thread name (just for debugging)
                                         (size_t) 1 * configMINIMAL_STACK_SIZE, // stack size
                                         (void *) NewBox,                       // passed as argument to task-function
                                         (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                         (xTaskHandle*) NULL                    // can return a handle to created task
                                         );
    }
    return &(NewBox->Box2Draw);
}
void tMoveBox(void *pvParameters) {

    drawTaskArgs_t* Arguments = (drawTaskArgs_t*) pvParameters;
    xQueueHandle DrawQueue = Arguments->DrawQueue;

    Assert(NULL != DrawQueue, ec_InvalidArgument);

    box2d_t* DrawBox = &(Arguments->Box2Draw);
    while (1) {

        int PosX = DrawBox->PosX;
        int PosY = DrawBox->PosY;
        int NewPosX = PosX + DrawBox->SpeedX;
        int NewPosY = PosY + DrawBox->SpeedY;

        int Border_Right = LCD_Width  - DrawBox->Width;
        int Border_Lower = LCD_Height - DrawBox->Height;

        if (NewPosX > Border_Right) {
            DrawBox->SpeedX = -DrawBox->SpeedX;
            int Excess = NewPosX - Border_Right;
            NewPosX -= Excess;
        }
        else if (NewPosX < 0) {
            DrawBox->SpeedX = -DrawBox->SpeedX;
            NewPosX = -NewPosX;
        }
        if (NewPosY > Border_Lower) {
            DrawBox->SpeedY = -DrawBox->SpeedY;
            int Excess = NewPosY - Border_Lower;
            NewPosY -= Excess;
        }
        else if (NewPosY < 0) {
            DrawBox->SpeedY = -DrawBox->SpeedY;
            NewPosY = -NewPosY;
        }

        if (1) { /* 1st implementation, clearing the previous box and redrawing all of it. */
            box2d_t ClearBox ={Color_Background,PosX,PosY,DrawBox->Width,DrawBox->Height,0,0};
            queueInsert(DrawQueue, &ClearBox);

            DrawBox->PosX = NewPosX;
            DrawBox->PosY = NewPosY;
            queueInsert(DrawQueue, DrawBox);
        }
        else { // 1nd implementation: clear as few pixels as possible
            // ToDo
        }

        mSleep(30);
    }
}
void queueInsert(xQueueHandle DrawQueue, box2d_t *Box) {

    while (xQueueSend(DrawQueue, Box, (portTickType) -1) != pdTRUE) 
      taskYIELD();

}
void tDrawRect(void *pvParameters) {
    xQueueHandle DrawQueue= (xQueueHandle) pvParameters;
    Assert(NULL != DrawQueue, ec_InvalidArgument);

    while (1) {
        box2d_t Box2Draw;
        while ( xQueueReceive(DrawQueue, &Box2Draw, (portTickType) -1) ) {
            LCD_DrawRect(Box2Draw.PosX,Box2Draw.PosY,Box2Draw.Width,Box2Draw.Height,Box2Draw.BoxColor);
        }
        taskYIELD();
    }
}

//} Function definitions
